package fpcs.core;

/**
 *
 * Abstract Classifier class
 *
 * @author Ignas Kukenys
 *
 */
public abstract class Classifier {

    protected Condition condition;
    protected int action;
//    protected double prediction;
//    protected double predictionError;
    protected double fitness;
    protected int numerosity;
    protected int experience;
    protected double actionSetSize;
    protected int timeStamp;

    protected double accuracy;
    protected int correctTrack;

    protected Classifier() {
        actionSetSize = 1;
    }

    protected Classifier(double actionSetSize, int timestamp) {

//        this.prediction = LearningParams.INITIAL_PREDICTION;
//        this.predictionError = LearningParams.INITIAL_PREDICTION_ERROR;
        this.fitness = LearningParams.INITIAL_FITNESS;

        this.numerosity = 1;
        this.experience = 0;
        this.correctTrack = 0;
        this.accuracy=0.0;

        this.actionSetSize = actionSetSize;
        this.timeStamp = timestamp;
    }

    protected Classifier(int action, double setSize, int timestamp) {
        this(setSize, timestamp);
        this.action = action;
    }

    public int getAction() {
        return action;
    }

    public void setAction(int action) {
        this.action = action;
    }

    public int getExperience() {
        return experience;
    }

    public void setExperience(int exp) {
        this.experience = exp;
    }

    public void increaseExperience() {
        experience++;
    }

//    public double getPrediction() {
//        return prediction;
//    }
//
//    public void setPrediction(double pre) {
//        prediction = pre;
//    }
//
//    public double getPredictionError() {
//        return predictionError;
//    }
//
//    public void setPredictionError(double preE) {
//        predictionError = preE;
//    }

    public double getFitness() {
        return fitness;
    }

    public void setFitness(double fit) {
        fitness = fit;
    }

    public int getNumerosity() {
        return numerosity;
    }

    public void addNumerosity(int num) {
        numerosity += num;
    }

    public int getTimeStamp() {
        return timeStamp;
    }

    public void setTimeStamp(int ts) {
        timeStamp = ts;
    }

    public Condition getCondition() {
        return condition;
    }

    public void setCondition(Condition condition) {
        this.condition = condition;
    }

    public boolean subsumes(Classifier cl) {
        return (cl.getAction() == action) && this.isSubsumer() && this.isMoreGeneral(cl);
    }

    public boolean isSubsumer() {
        if (experience > LearningParams.SUBSUMPTION_EXP_THRESHOLD_THETA_SUB ) {
//            && predictionError < LearningParams.ACCURACY_THRESHOLD_EPSILON_0
            return true;
        }
        return false;
    }

    public boolean isMoreGeneral(Classifier cl) {
        return condition.isMoreGeneral(cl.getCondition());
    }

    public double getDeletionVote(double meanFitness) {

//        if self.fitness/self.numerosity >= cons.delta*meanFitness or self.experience < cons.theta_del:
//    	    return self.actionSetSize*self.numerosity
//        # fitness was becoming zero
//        if self.fitness == 0.0:
//            return self.actionSetSize * self.numerosity * meanFitness / (cons.fitnessIni/self.numerosity)
//        else:
//            return self.actionSetSize * self.numerosity * meanFitness / (self.fitness/self.numerosity)
//
//
//

        double deletionVote = 0;
        if (getFitness() / numerosity >= LearningParams.DELETION_FRACTION_DELTA * meanFitness || experience < LearningParams.DELETION_FITNESS_THRESHOLD_THETA_DEL) {
            deletionVote = actionSetSize * numerosity;
        }
        if (getFitness() == 0.0){
            deletionVote=actionSetSize * numerosity * meanFitness / (LearningParams.INITIAL_FITNESS /numerosity);
        }
        else
        deletionVote = actionSetSize * numerosity * meanFitness / (getFitness() / numerosity);

        return deletionVote;
    }
    

    public boolean match(State state) {
        return condition.matches(state);
    }

//    public double updatePrediction(double P) {
//        if ((double) experience < 1.0 / LearningParams.LEARNING_RATE_BETA) {
//            prediction = (prediction * ((double) experience - 1.) + P) / (double) experience;
//        } else {
//            //System.out.println("LEARNING_RATE_BETA: " + LearningParams.LEARNING_RATE_BETA);
//            prediction += LearningParams.LEARNING_RATE_BETA * (P - prediction);
//        }
//        return prediction * numerosity;
//    }

//    public double updatePreError(double P) {
//        if ((double) experience < 1.0 / LearningParams.LEARNING_RATE_BETA) {
//            predictionError = (predictionError * ((double) experience - 1.) + Math.abs(P - prediction)) / (double) experience;
//        } else {
//            predictionError += LearningParams.LEARNING_RATE_BETA * (Math.abs(P - prediction) - predictionError);
//        }
//        return predictionError * numerosity;
//    }

    public double getAccuracy() {
//        double accuracyy;

//        if (predictionError <= (double) LearningParams.ACCURACY_THRESHOLD_EPSILON_0) {
//            accuracyy = 1.0;
//        } else {
//            accuracyy = LearningParams.FITNESS_FALLOFF_ALPHA * Math.pow(predictionError / LearningParams.ACCURACY_THRESHOLD_EPSILON_0, -LearningParams.FITNESS_EXPONENT_NU);
//        }
        return accuracy;
    }

        public double updateFitness() {
//        fitness += LearningParams.LEARNING_RATE_BETA * ((accuracy * numerosity) / accSum - fitness);
            fitness=Math.pow(accuracy,LearningParams.ACCURACY_THRESHOLD_EPSILON_0);
        return getFitness();//fitness already considers numerosity
    }

    public double updateActionSetSize(double numerositySum) {
        if (experience < 1.0 / LearningParams.LEARNING_RATE_BETA) {
            actionSetSize = (actionSetSize * (double) (experience - 1.0) + numerositySum) / (double) experience;
        } else {
            actionSetSize += LearningParams.LEARNING_RATE_BETA * (numerositySum - actionSetSize);
        }
        return actionSetSize * numerosity;
    }

    protected boolean mutateAction(int numberOfActions) {
        boolean changed = false;

        if (RandomUtils.random() < LearningParams.PROB_MUTATION_MU) {
            int act = 0;
            do {
                act = (int) (RandomUtils.random() * numberOfActions);
            } while (act == action);
            action = act;
            changed = true;
        }
        return changed;
    }


    public void increaseCorrectTrack() {
        this.correctTrack++;
    }

    public void updateAccuracy(){        
        this.accuracy = this.correctTrack / (double) this.experience;
    }

    public void setAccuracy(double accuracy) {
        this.accuracy = accuracy;
    }

    public String toString() {
        return String.format("X%6d N%3d F%1.4f A%1.4f %s -> %d",
                this.experience,
                this.numerosity,
                this.getFitness(),
//                this.getPrediction(),
                this.getAccuracy(),
                this.getCondition(),
                this.getAction());
    }

    public abstract Classifier offspring();

    public abstract boolean applyMutation(State state, int numberOfActions);

    public abstract boolean isIdentical(Classifier cl);

    public abstract boolean applyCrossover(Classifier cl);
}
