package fpcs.core;

/**
 * This class generates a prediction array of the provided set.
 * The prediction array is generated according to Wilson's Classifier Fitness Based on Accuracy 
 * (Evolutionary Computation Journal, 1995).
 * Moreover, this class provides all methods to handle selection in the prediction array, essentially, to select the
 * best action, a present random action or an action by roulette wheel selection. 
 *
 * @author    Martin V. Butz
 * @version   XCSJava 1.0
 * @since     JDK1.1
 */
public class PredictionArray {

    private double votes[];

    public PredictionArray(ClassifierSet set, int size) {
        votes = new double[size];
        for (int i = 0; i < set.size(); i++) {
            Classifier cl = set.get(i);
            votes[cl.getAction()] += cl.getFitness();
        }
    }

    public PredictionArray(ClassifierSet set, int size, int minVotingAge) {

        votes = new double[size];
        for (Classifier cl : set) {
            if (cl.getExperience() >= minVotingAge) {
                votes[cl.getAction()] += cl.getFitness();
            

                //assuming the first index holds 0,2,3,4,5 and the second index 1,7,8,9,6
//                if (cl.getAction() == 0 || cl.getAction() == 2 || cl.getAction() == 3 || cl.getAction() == 4 || cl.getAction() == 5) {
//                    votes[cl.getAction()] += cl.getFitness();
//                } else {
//                    votes[cl.getAction()] += cl.getFitness();
//                }
            }
        }
    }

//    public double[] getPredictions() {
//        return votes;
//    }

//        public int randomActionWinner() {
//        int ret = 0;
//        do {
//            ret = (int) (RandomUtils.random() * votes.length);
//        } while (votes[ret] == 0);
//        return ret;
//    }

    /**
     * Returns the highest value in the prediction array.
     */
    public double getBestValue() {
        int i;
        double max;
        for (i = 1, max = votes[0]; i < votes.length; i++) {
            if (max < votes[i]) {
                max = votes[i];
            }
        }
        return max;
    }

    /**
     * Returns the value of the specified entry in the prediction array.
     */
//    public double getValue(int i) {
//        if (i >= 0 && i < votes.length) {
//            return votes[i];
//        }
//        return -1.;
//    }
    /**
     * Selects the action in the prediction array with the best value.
     */
    public int bestActionWinner() {
      int ret = 0;
        for (int i = 1; i < votes.length; i++) {
            if (votes[ret] < votes[i]) {
                ret = i;
            }
        }
        return ret;
//        if (votes[0] > votes[1]) {
//            if (actualAction == 0 || actualAction == 1) {
//                return 0;
//            } else if (actualAction == 2 || actualAction == 7) {
//                return 2;
//            } else if (actualAction == 3 || actualAction == 8) {
//                return 3;
//            } else if (actualAction == 4 || actualAction == 9) {
//                return 4;
//            } else {
//                return 5;
//            }
//
//        } else {
//
//            if (actualAction == 0 || actualAction == 1) {
//                return 1;
//            } else if (actualAction == 2 || actualAction == 7) {
//                return 7;
//            } else if (actualAction == 3 || actualAction == 8) {
//                return 8;
//            } else if (actualAction == 4 || actualAction == 9) {
//                return 9;
//            } else {
//                return 6;
//            }
//        }
    }

    @Override
    public String toString() {
        System.out.println("PredictionArray: ");
        for (int i = 0; i
                < votes.length; i++) {
            System.out.print(votes[i] + ",");
        }


        return super.toString();

    }
}
