package fpcs.image;

import fpcs.core.Classifier;
import fpcs.core.State;

/**
 *
 * @author Ignas Kukenys
 *
 */
public class ImageClassifier extends Classifier {

    //public static boolean DO_INVERSION = false;
    public ImageClassifier() {
    }

    public ImageClassifier(State situation, int action, double setSize, int timestamp) {
        super(action, setSize, timestamp);
        this.condition = FeatureCondition.generateForState((ImageState) situation);
        assert (((FeatureCondition) this.condition).getHaarRangeFeatures().size() > 0);
    }

    private ImageClassifier(ImageClassifier copy) {
        if (copy.numerosity == 0) {
            throw new RuntimeException("Cannot clone classifier, numerosity = 0");
        }

        condition = copy.getCondition().clone();
        this.action = copy.getAction();
        this.fitness = copy.fitness / copy.numerosity;
        this.numerosity = 1;
        this.experience = 0;
        this.actionSetSize = copy.actionSetSize;
        this.timeStamp = copy.timeStamp;
    }

    protected boolean mutateCondition(ImageState state) {
        return getCondition().mutate(state);
    }

    @Override
    public boolean applyCrossover(Classifier cl) {
        return getCondition().crossover(cl.getCondition());
    }

    @Override
    public ImageClassifier offspring() {
        return new ImageClassifier(this);
    }

    @Override
    public boolean applyMutation(State state, int numberOfActions) {
        mutateAction(numberOfActions);
        return mutateCondition((ImageState) state);
    }

    @Override
    public boolean isIdentical(Classifier cl) {
        if (cl instanceof ImageClassifier) {
            ImageClassifier icl = (ImageClassifier) cl;
            return (this.getAction() == icl.getAction()) && this.getCondition().isIdentical(cl.getCondition());
        }
        return false;
    }

    @Override
    public boolean isMoreGeneral(Classifier cl) {
        return ((FeatureCondition) condition).isMoreGeneral((FeatureCondition) cl.getCondition());
    }

    @Override
    public FeatureCondition getCondition() {
        return (FeatureCondition) condition;
    }
}
