package fpcs.symbolic;

import fpcs.core.Condition;
import fpcs.core.LearningParams;
import fpcs.core.State;

/**
 *
 * @author Ignas Kukenys
 *
 */
public class SymbolicCondition extends Condition {

    private String condition;

    public SymbolicCondition(String condition) {
        this.condition = condition;
    }

    public SymbolicCondition(char[] condition) {
        this.condition = new String(condition);
    }

    public String getConditionAsString() {
        return condition;
    }

    @Override
    public boolean matches(State state) {

        String stateString = ((SymbolicState) state).getStateAsString();

        if (condition.length() != stateString.length()) {
            return false;
        }
        for (int i = 0; i < condition.length(); i++) {
            if (condition.charAt(i) != SymbolicClassifier.SYMBOL_DONTCARE && condition.charAt(i) != stateString.charAt(i)) {
                return false;
            }
        }
        return true;

    }

    @Override
    public boolean isMoreGeneral(Condition cond) {
        boolean ret = false;
        String condString = ((SymbolicCondition)cond).getConditionAsString();
        int length = condition.length();
        for (int i = 0; i < length; i++) {
            if (condition.charAt(i) != SymbolicClassifier.SYMBOL_DONTCARE && condition.charAt(i) != condString.charAt(i)) {
                return false;
            } else if (condition.charAt(i) != condString.charAt(i)) {
                ret = true;
            }
        }
        return ret;
    }

    @Override
    public String toString() {
        return condition;
    }

    @Override
    public SymbolicCondition clone() {
        return new SymbolicCondition(this.condition);
    }

    @Override
    public boolean isIdentical(Condition cond) {
        if (cond instanceof SymbolicCondition) {
            return ((SymbolicCondition)cond).condition.equals(this.condition);
        }
        return false;
    }

}
