package fpcs.symbolic.patterns;

import java.awt.Dimension;
import java.awt.image.BufferedImage;
import fpcs.utils.IDXFileReader;
import fpcs.utils.ImageUtils;

/**
 *
 * @author Ignas Kukenys
 *
 */

public class MNISTSymbolicPatternProvider extends SymbolicPatternProvider {

    public static final int GREY_THRESHOLD = 127;
    public static final int MNIST_IDX_LABELS_DIM = 1;
    public static final int MNIST_IDX_IMAGES_DIM = 3;


    private final int patternSize = 36; // 28 * 28 => 6 * 6
    private final char[] data;
    private final byte[] labels;

    public MNISTSymbolicPatternProvider(String dataFile, String labelFile) {
        byte[] tmpdata = IDXFileReader.readAllBytes(dataFile, MNIST_IDX_IMAGES_DIM);
        byte[] buffer = new byte[28 * 28];
        Dimension dim = new Dimension(28, 28);

        int nexamples = tmpdata.length / buffer.length;
        data = new char[nexamples * patternSize];

        for (int i = 0; i < nexamples; i++) {
            System.arraycopy(tmpdata, i * buffer.length, buffer, 0, buffer.length);
            BufferedImage im1 = ImageUtils.bytearray2image(buffer, dim);
            BufferedImage im2 = ImageUtils.getScaledVersion(im1.getSubimage(2, 2, 24, 24), 6, 6);
            double[] samples = ImageUtils.image2array(im2);

            for(int j = 0; j < samples.length; j++) {
                if (samples[j] < GREY_THRESHOLD) {
                    data[i*patternSize + j] = '1';
                } else {
                    data[i*patternSize + j] = '0';
                }
            }
        }
        
        
        labels = IDXFileReader.readAllBytes(labelFile, MNIST_IDX_LABELS_DIM);
    }


    @Override
    public int size() {
        assert(data.length == labels.length);
        return data.length / patternSize;
    }

    @Override
    public char[] getPattern(int index, char[] array) {

        if (array == null || array.length < patternSize) {
            array = new char[patternSize];
        }

        System.arraycopy(data, index * patternSize, array, 0, patternSize);

        return array;
    }

    @Override
    public int getClassLabel(int index) {
        return labels[index];
    }

    @Override
    public int getPatternSize() {
        return patternSize;
    }

}
