package problems.mnist;

import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import fpcs.image.ImageState;
import fpcs.utils.IDXFileReader;
import fpcs.utils.ImageUtils;

/**
 *
 * @author Ignas Kukenys
 *
 */

public class MNISTImagePatternProvider implements Iterable<ImageState> {

    public static final int GREY_THRESHOLD = 127;
    public static final int MNIST_IDX_LABELS_DIM = 1;
    public static final int MNIST_IDX_IMAGES_DIM = 3;
    public static final int MNIST_IMAGE_WIDTH = 28;
    public static final int MNIST_IMAGE_HEIGHT = 28;
    public static final int MNIST_NUM_LABELS = 10;

    public static final int MNIST_CROP_X = 0;
    public static final int MNIST_CROP_Y = 0;
    public static final int MNIST_CROP_WIDTH = 28;
    public static final int MNIST_CROP_HEIGHT = 28;

    private int targetWidth = 28, targetHeight = 28;


    private List<ImageState> states = new ArrayList<ImageState>();
    private List<Integer> labels = new ArrayList<Integer>();

    private int numActions;

    public MNISTImagePatternProvider(String dataFile, String labelFile) {
        this(dataFile, labelFile, null);
    }

    public MNISTImagePatternProvider(String dataFile, String labelFile, int[] includeLabels) {
        byte[] labelData = IDXFileReader.readAllBytes(labelFile, MNIST_IDX_LABELS_DIM);
        
        byte[] imageData = IDXFileReader.readAllBytes(dataFile, MNIST_IDX_IMAGES_DIM);
        byte[] buffer = new byte[MNIST_IMAGE_WIDTH * MNIST_IMAGE_HEIGHT];
        Dimension dim = new Dimension(MNIST_IMAGE_WIDTH, MNIST_IMAGE_HEIGHT);

        int[] histo = new int[MNIST_NUM_LABELS];

        int nexamples = imageData.length / buffer.length;

        Set<Integer> include = new HashSet<Integer>();
        
        if (includeLabels != null) {
            for(int i = 0; i < includeLabels.length; i++) {
                if (!include.contains(includeLabels[i])) {
                    include.add(includeLabels[i]);
                }
            }
        }

        numActions = MNIST_NUM_LABELS;

        for (int i = 0; i < nexamples; i++) {

            histo[labelData[i]]++;

            if (includeLabels == null || include.contains((int)(labelData[i] & 0xFF))) {

                System.arraycopy(imageData, i * buffer.length, buffer, 0, buffer.length);

                BufferedImage im1 = ImageUtils.bytearray2image(buffer, dim);
                BufferedImage im2 = ImageUtils.getScaledVersion(im1.getSubimage(MNIST_CROP_X, MNIST_CROP_Y, MNIST_CROP_WIDTH, MNIST_CROP_HEIGHT), targetWidth, targetHeight);
                double[] samples = ImageUtils.image2array(im2);

                /*
                for(int j = 0; j < samples.length; j++) {
                    if (samples[j] < GREY_THRESHOLD) {
                        samples[j] = 100;
                    } else {
                        samples[j] = 0;
                    }
                }*/

                states.add(new ImageState(targetWidth, targetHeight, samples));
                labels.add(labelData[i] & 0xFF);

            }

        }
        
        System.out.println("Training data labels");
        for (int i = 0; i < histo.length; i++) {
            if (includeLabels == null || include.contains(i)) {
                System.out.println("Label: " + i + " Examples: " + histo[i]);
            }
        }

        
    }


    @Override
    public Iterator<ImageState> iterator() {
        return states.iterator();
    }

    public Iterable<ImageState> stateIterator() {
        return this;
    }

    public Iterable<Integer> labelIterator() {
        return labels;
    }

    public int size() {
        return states.size();
    }

    public ImageState getPattern(int index) {
        return states.get(index);
    }

    public int getLabel(int index) {
        return labels.get(index);
    }

    public int getNumActions() {
        return numActions;
    }


}
