package problems.mnist;

import fpcs.core.Environment;
import fpcs.core.ClassifierFactory;
import fpcs.core.RandomUtils;
import fpcs.core.State;
import fpcs.image.ImageClassifierFactory;

/**
 *
 * @author Ignas Kukenys
 *
 */

public class MNISTIncrementalEnvironment extends Environment {

    private MNISTImagePatternProvider patterns;

    private int count = 0;
    private int duration = 0;
    private int numLabels = 1;
    

    public MNISTIncrementalEnvironment(MNISTImagePatternProvider patterns, int duration) {
        this.patterns = patterns;
        this.duration = duration;
    }

    @Override
    public State resetState() {

        if (count % (duration / 10) == 0) {
            numLabels++;
        }

        do {
            int dice = (int)(patterns.size() * RandomUtils.random());
    
            currentState = patterns.getPattern(dice);
            currentClass = patterns.getLabel(dice);
        } while (currentClass >= numLabels);


        count++;
        return currentState;
    }

    @Override
    public int getNumActions() {
        return patterns.getNumActions();
    }

    public ClassifierFactory getClassifierFactory() {
        return ImageClassifierFactory.getInstance();
    }


}
