package examples;

import java.io.File;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;
import fpcs.core.ClassifierSet;
import fpcs.core.Environment;
import fpcs.core.HFPCS_Supervised;
import fpcs.core.LearningParams;
import fpcs.image.ImageClassifierFactory;
import fpcs.listeners.ConfusionMatrixListener;

import fpcs.listeners.IExploitListener;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import problems.mnist.MNISTImagePatternEnvironment;
import problems.mnist.MNISTImagePatternProvider;

/**
 *
 * @author Ignas Kukenys
 *
 */
public class MNIST_DivConq_Grid {

    public static void main(String[] args) {

      String jobid="000";

        LearningParams.NUM_GENERATIONS=4000001;
       LearningParams.MAX_SUBPOPULATION_SIZE=60000;
       LearningParams.MAX_POPULATION_SIZE=12000;
       double confusionMatrixStart =0.95;
         int[] digits = new int[]{0, 1, 2, 3, 4, 5, 6, 7, 8, 9};

        DateFormat timeformat = new SimpleDateFormat("HH:mm:ss");
        timeformat.setTimeZone(TimeZone.getTimeZone("GMT"));

        System.out.println(System.getProperty("user.dir"));

        String path = "d:/";
//        String jobid = System.getenv().containsKey("JOB_ID") ? System.getenv().get("JOB_ID") : "000000";
//        String path = "/local/tmp/tebadi/" + jobid + "/";

        String data = "data/raw/train-images.idx3-ubyte";
        String labels = "data/raw/train-labels.idx1-ubyte";

        String testdata = "data/raw/t10k-images.idx3-ubyte";
        String testlabels = "data/raw/t10k-labels.idx1-ubyte";

//        String experimentPath = path + "/experiments/" + date + "-" + "fpcs-ucs-dc" + "/";
//        (new File(experimentPath)).mkdirs();

       
        System.out.println("Population: " + LearningParams.MAX_POPULATION_SIZE);
        System.out.println("Subpopulation: " + LearningParams.MAX_SUBPOPULATION_SIZE);
        System.out.println("Max Generations: " + LearningParams.NUM_GENERATIONS);






        MNISTImagePatternProvider patterns = new MNISTImagePatternProvider(path + data, path + labels);

        Environment env = new MNISTImagePatternEnvironment(patterns);

        long start = System.currentTimeMillis();
        HFPCS_Supervised agent = new HFPCS_Supervised(env);

         agent.setClassifierFactory(ImageClassifierFactory.getInstance());

//        agent.addExploitListener(new IExploitListener(5000,5000));
         agent.addListener(new IExploitListener(5000,5000));
         ConfusionMatrixListener cm = new ConfusionMatrixListener(digits.length, (int) (LearningParams.NUM_GENERATIONS * confusionMatrixStart));
         agent.addListener(cm);

        
        redirectOutput(path, "x-" + jobid + "-classifiers.txt");
        ClassifierSet runset = agent.run();
        runset.printAllByFitness(runset.size());

//        ClassifierSet[] subpopulation=agent.getSubpopulation();
//
//
//            redirectOutput(path, "x-" + jobid + "-subpopulation"+01+".txt");
//            ClassifierSet set01=subpopulation[0];
//            set01.printAllByFitness(set01.size());
//
//            redirectOutput(path, "x-" + jobid + "-subpopulation"+27+".txt");
//            ClassifierSet set27=subpopulation[2];
//            set27.printAllByFitness(set27.size());
//
//            redirectOutput(path, "x-" + jobid + "-subpopulation"+38+".txt");
//            ClassifierSet set38=subpopulation[3];
//            set38.printAllByFitness(set38.size());
//
//            redirectOutput(path, "x-" + jobid + "-subpopulation"+49+".txt");
//            ClassifierSet set49=subpopulation[4];
//            set49.printAllByFitness(set49.size());
//
//            redirectOutput(path, "x-" + jobid + "-subpopulation"+56+".txt");
//            ClassifierSet set56=subpopulation[5];
//            set56.printAllByFitness(set56.size());
//
//            redirectOutput(path, "x-" + jobid + "-cm-training.txt");
//        cm.printMatrix();

        long end = System.currentTimeMillis();
        redirectOutput(path, "x-" + jobid + "-trainingTime.txt");
        System.out.println("Testing time " + timeformat.format(new Date(end - start)));

        int correct = 0;

        MNISTImagePatternProvider testPatterns = new MNISTImagePatternProvider(path + testdata, path + testlabels,digits);

        ConfusionMatrixListener cm2 = new ConfusionMatrixListener(testPatterns.getNumActions());

        for (int i = 0; i < testPatterns.size(); i++) {

            int agentAction = agent.evaluate(testPatterns.getPattern(i));
            cm2.recordAction(testPatterns.getLabel(i), agentAction);

            if (agentAction == testPatterns.getLabel(i)) {
                correct++;
            }
        }

        System.out.println("Evalset classification rate: " + correct / (double) testPatterns.size());

        cm2.printMatrix();

        String timetaken = timeformat.format(new Date(end - start));


        System.out.println("----Experiment complete! Taken: " + timetaken);

        System.gc();

    }

      public static void redirectOutput(String path, String filename) {
        try {
            System.setOut(new PrintStream(new File(path + filename)));
        } catch (FileNotFoundException ex) {
            throw new RuntimeException("Unable to redirect stdout to: " + path + filename, ex);
        }
    }
}
