package fpcs.core;

import fpcs.listeners.IExploitListener;
import fpcs.symbolic.SymbolicClassifierFactory;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author  Ignas Kukenys
 * 
 */
public class HFPCS_Supervised {

    private Environment environment;
    private List<ITrainingListener > listeners = new ArrayList<ITrainingListener>();
    private ClassifierSet population = new ClassifierSet();
    private State state;
//    private ClassifierSet[] subpop;
    private ClassifierFactory classifierFactory = SymbolicClassifierFactory.getInstance();
    static final int[] pair = new int[]{1, 0, 7, 8, 9, 6, 5, 2, 3, 4};
    private int numActions = 1;

    public HFPCS_Supervised(Environment env) {
        environment = env;

//        subpop = new ClassifierSet[10];
//        subpop[0] = subpop[1] = new ClassifierSet();
//        subpop[2] = subpop[7] = new ClassifierSet();
//        subpop[3] = subpop[8] = new ClassifierSet();
//        subpop[4] = subpop[9] = new ClassifierSet();
//        subpop[5] = subpop[6] = new ClassifierSet();

    }

   public void addListener(ITrainingListener listener){
       listeners.add(listener);
   }


    public ClassifierSet run() {
        if (listeners.isEmpty()) {
            Logger.getLogger(HFPCS_Supervised.class.getName()).log(Level.WARNING, "Warning: no listeners set on the LPCS object");
        }

        for (int generation = 0; generation < LearningParams.NUM_GENERATIONS; generation++) {
            state = environment.resetState();
            explore(population, state, generation,LearningParams.MAX_POPULATION_SIZE);

//            int action = environment.getCorrectAction();
//            explore(subpop[action], state, generation,LearningParams.MAX_SUBPOPULATION_SIZE);

            state = environment.resetState();
            exploit(population, state, generation);
        }

        return population;
    }

    protected void explore(ClassifierSet population, State state, int generation, int maxPopulationSize) {

        this.numActions = environment.getNumActions();
        int actualAction = environment.getCorrectAction();

        ClassifierSet matchSet = makeCoveringMatchSet(population, state, generation, actualAction,maxPopulationSize);
        ClassifierSet correctSet = makeCorrectSet(matchSet, actualAction);
        correctSet.updateCorrectTrack();
        matchSet.updateSet();

        if (LearningParams.FLAG_ACTION_SET_SUBSUMPTION) {
            doActionSetSubsumption(population, matchSet);
        }

        runGA(population, correctSet, generation, state, environment.getNumActions());


    }

    protected void exploit(ClassifierSet population, State state, int generation) {

        ClassifierSet matchSet = makeMatchSet(population, state);
        PredictionArray predictionArray = new PredictionArray(matchSet, numActions, LearningParams.MIN_VOTING_AGE);
        int actionWinner = predictionArray.bestActionWinner(); //

//        if (population == this.population) {
//            exploit(subpop[actionWinner], state, generation);
//
//        } else {
            double prediction = predictionArray.getBestValue();
            double reward = environment.executeAction(actionWinner);
            int correctAction = environment.getCorrectAction();

            boolean wasCorrect = actionWinner == correctAction; 

//            if (!wasCorrect) {
//                makeCovering(state, actionWinner);
//            }

            notifyListeners(generation, state, correctAction, population, matchSet, actionWinner, prediction, reward, wasCorrect);
//        }

    }

    protected void notifyListeners(int generation, State state, int stateAction, ClassifierSet population, ClassifierSet matchSet, int actionWinner, double prediction, double reward, boolean wasCorrect) {
        for (ITrainingListener  listener : listeners) {
            listener.onExploit(generation, state, stateAction, population, matchSet, actionWinner, prediction, reward, wasCorrect);
        }
    }

    /*
    
     */
    protected ClassifierSet makeCoveringMatchSet(ClassifierSet population, State state, int timestamp, int actualAction,int maxPopulationSize) {

        ClassifierSet matches = new ClassifierSet();


        boolean actionCovered = false;

        for (int i = 0; i < population.size(); i++) {
            Classifier classifier = population.get(i);
            if (classifier.match(state)) {
                matches.add(classifier);
                if (classifier.getAction() == actualAction) {
                    actionCovered = true;
                }


            }
        }

        boolean again = true;

        while (again) {
            again = false;

            if (!actionCovered) {
                Classifier newCl = classifierFactory.createForStateAndAction(state, actualAction, population.getNumerositySum() + 1.0, timestamp);
                population.add(newCl);
                matches.add(newCl);

            }

            while (population.getNumerositySum() > maxPopulationSize-1) {
                Classifier toRemove = population.selectForDeletionRW();


                if (toRemove != null) {
                    toRemove.addNumerosity(-1);

                    population.addNumerositySum(-1);
                    population.removeEmpty(toRemove);

                    if (matches.contains(toRemove)) {
                        matches.addNumerositySum(-1);
                        matches.removeEmpty(toRemove);



                        if (!matches.contains(toRemove) && !matches.isActionCovered(toRemove.getAction())) {
//                            actionCovered[toRemove.getAction()] = false;
                            again = true;


                        }
                    }

                }
            }
        }
        return matches;


    }

    protected static ClassifierSet makeMatchSet(ClassifierSet population, State state) {

        ClassifierSet matches = new ClassifierSet();

        for (int i = 0; i < population.size(); i++) {
            Classifier classifier = population.get(i);
            if (classifier.match(state)) {
                matches.add(classifier);
            }
        }

        return matches;

    }

    protected static ClassifierSet makeActionSet(ClassifierSet matchSet, int action) {

        ClassifierSet actions = new ClassifierSet();

        for (int i = 0; i < matchSet.size(); i++) {
            Classifier classifier = matchSet.get(i);
            if (classifier.getAction() == action) {
                actions.add(classifier);
            }
        }
        return actions;

    }

    protected void runGA(ClassifierSet population, ClassifierSet actionSet, int timestamp, State state, int numActions) {
        if (actionSet.size() == 0 || timestamp - actionSet.getTimeStampAverage() < LearningParams.GA_THRESHOLD_THETA_GA) {
            return;

        }
        actionSet.setTimeStamps(timestamp);



        double fitSum = actionSet.getFitnessSum();

        // Select two Classifiers with roulette Wheel Selection
        Classifier cl1P = LearningParams.FLAG_TOURNAMENT_SELECTION ? actionSet.selectForBreedingT() : actionSet.selectForBreedingRW(fitSum);
        Classifier cl2P = LearningParams.FLAG_TOURNAMENT_SELECTION ? actionSet.selectForBreedingT() : actionSet.selectForBreedingRW(fitSum);

        Classifier cl1 = cl1P.offspring();
        Classifier cl2 = cl2P.offspring();

        // crossover and mutate
        boolean changed = cl1.applyCrossover(cl2);

        if (changed) {
            cl1.setAccuracy(cl1.getAccuracy() + cl2.getAccuracy() / 2.0);
            cl1.setFitness(LearningParams.FITNESS_REDUCTION * (cl1.getFitness() + cl2.getFitness()) / 2.0);
            cl2.setAccuracy(cl1.getAccuracy());
            cl2.setFitness(cl1.getFitness());
        } else {
            cl1.setFitness(LearningParams.FITNESS_REDUCTION * cl1.getFitness());
        }
        cl2.setFitness(LearningParams.FITNESS_REDUCTION * cl2.getFitness());

        boolean valid1 = cl1.applyMutation(state, numActions);
        boolean valid2 = cl2.applyMutation(state, numActions);




        /*
         * If evolution produces invalid individuals, increase the numerosity of the parents instead
         */



        if (valid1 && LearningParams.FLAG_GA_SUBSUMPTION) {
            subsumeClassifier(population, actionSet, cl1, cl1P, cl2P);


        } else if (valid1) {
            population.addSubsume(cl1);


        } else {
            cl1P.addNumerosity(1);
            population.addNumerositySum(1);


        }

        if (valid2 && LearningParams.FLAG_GA_SUBSUMPTION) {
            subsumeClassifier(population, actionSet, cl2, cl1P, cl2P);


        } else if (valid2) {
            population.addSubsume(cl2);


        } else {
            cl2P.addNumerosity(1);
            population.addNumerositySum(1);


        }

        population.reduceToSize(LearningParams.MAX_POPULATION_SIZE);

    }

    protected void doActionSetSubsumption(ClassifierSet population, ClassifierSet actionSet) {
        Classifier subsumer = null;

        for (Classifier classifier : actionSet) {
            if (classifier.isSubsumer()) {
                if (subsumer == null || classifier.isMoreGeneral(subsumer)) {
                    subsumer = classifier;
                }
            }
        }

        //If a subsumer was found, subsume all more specific classifiers in the action set
        if (subsumer != null) {
            ArrayList<Classifier> toRemove = new ArrayList<Classifier>();
            for (Classifier classifier : actionSet) {
                if (subsumer.isMoreGeneral(classifier)) {
                    int num = classifier.getNumerosity();
                    subsumer.addNumerosity(+num);
                    classifier.addNumerosity(-num);
                    toRemove.add(classifier);
                    population.removeEmpty(classifier);
                }
            }
            for (Classifier rm : toRemove) {
                actionSet.removeEmpty(rm);
            }
        }
    }

    protected void subsumeClassifier(ClassifierSet population, ClassifierSet actionSet, Classifier cl, Classifier cl1P, Classifier cl2P) {
        if (cl1P != null && cl1P.subsumes(cl)) {
            cl1P.addNumerosity(+1);
            population.addNumerositySum(+1);
        } else if (cl2P != null && cl2P.subsumes(cl)) {
            cl2P.addNumerosity(+1);
            population.addNumerositySum(+1);
        } else if (actionSet.subsumeClassifier(cl)) {
            population.addNumerositySum(+1);
        } else {
            population.addSubsume(cl);
        }
    }

    protected ClassifierSet makeCorrectSet(ClassifierSet matchSet, int action) {

        ClassifierSet correctset = new ClassifierSet();

        for (int i = 0; i < matchSet.size(); i++) {
            Classifier classifier = matchSet.get(i);
            if (classifier.getAction() == action) {
                correctset.add(classifier);
            }
        }
        return correctset;

    }

    protected void makeCovering(State state, int actualAction,int generation) {
        Classifier newCl = classifierFactory.createForStateAndAction(state, actualAction, population.getNumerositySum() + 1.0, generation);
        population.add(newCl);

        while (population.getNumerositySum() > LearningParams.MAX_POPULATION_SIZE - 1) {

            Classifier toRemove = population.selectForDeletionRW();

            if (toRemove != null) {
                toRemove.addNumerosity(-1);

                population.addNumerositySum(-1);
                population.removeEmpty(toRemove);

            }
        }

    }

    public synchronized void setClassifierFactory(ClassifierFactory factory) {
        this.classifierFactory = factory;

    }
//
//    public ClassifierSet[] getSubpopulation(){
//        return this.subpop;
//    }

    public int evaluate(State state) {
        ClassifierSet matchSet = makeMatchSet(population, state); // strict match set, no covering
        PredictionArray predictionArray = new PredictionArray(matchSet, environment.getNumActions(), LearningParams.MIN_VOTING_AGE); //, environment.getMaxPayoff());
        int actionWinner = predictionArray.bestActionWinner();

//        ClassifierSet matchSubSet = makeMatchSet(subpop[actionWinner], state);
//        PredictionArray predictionSubArray = new PredictionArray(matchSubSet, environment.getNumActions(), LearningParams.MIN_VOTING_AGE);
//        int actionSubWinner = predictionSubArray.bestActionWinner();
//
//        return actionSubWinner;
        return  actionWinner;
    }
}
