package fpcs.core;

public class LearningParams {

    /**
     * Max features in messy encoding
     */
    public static final int MESSY_MAX_FEATURES = 10;

    /**
     * Probability of losing features in messy encoding
     */
    public static final double MESSY_PROB_SHRINK = 0.005;

    /**
     * Probability of adding features in messy encoding
     */
    public static final double MESSY_PROB_EXPAND = 0.005;


    /**
     * Payoff for correct classification
     */
    public static double MAX_PAYOFF = 1000;


    /**
     * Default number of generations. Number of problem instances to explore
     */
    public static int NUM_GENERATIONS = 20000;

    /**
     * Max population size. Max number of micro-classifiers. Numerosity sum cap.
     */
    public static int MAX_POPULATION_SIZE = 5000;

    public static int MAX_SUBPOPULATION_SIZE=12000;


    /**
     * Minimum voting age for classifiers to participate in exploit steps
     */
    public static int MIN_VOTING_AGE = 2;


    /**
     * A flag to perform subsumption after GA
     */
    public static boolean FLAG_GA_SUBSUMPTION = true;

    /**
     * A flag to use tournament selection instead of roulette-wheel
     */
    public static boolean FLAG_TOURNAMENT_SELECTION = true;

    /**
     * Fraction of classifiers participating in tournament selection
     */
    public static double TOURNAMENT_FRACTION_TAU = 0.8;

    /**
     * The fall of rate in the fitness evaluation.
     */
    public static double FITNESS_FALLOFF_ALPHA = 0.1; // original 0.1

    /**
     * The learning rate for updating fitness, prediction, prediction error, 
     * and action set size estimate in XCS's classifiers.
     */
    public static double LEARNING_RATE_BETA = 0.2;

    /**
     * The fraction of the mean fitness of the population below which the fitness of a classifier may be considered 
     * in its vote for deletion.
     */
    public static double DELETION_FRACTION_DELTA = 0.1;

    /**
     * Specifies the exponent in the power function for the fitness evaluation.
     */
    public static double FITNESS_EXPONENT_NU = 5.0;

    /**
     * The threshold for the GA application in an action set.
     */
    public static double GA_THRESHOLD_THETA_GA = 25; // default: 25

    /**
     * The error threshold under which the accuracy of a classifier is set to one.
     */
    public static double ACCURACY_THRESHOLD_EPSILON_0 = 10; // TODO: this could be percentage of MaxPayoff? was: 10

    /**
     * Specified the threshold over which the fitness of a classifier may be considered in its deletion probability.
     */
    public static int DELETION_FITNESS_THRESHOLD_THETA_DEL = 20;

    /**
     * The probability of applying crossover in an offspring classifier.
     */
    public static double PROB_CROSSOVER_CHI = 0.8;


    /**
     * The probability of applying crossover for each allele (uniform crossover)
     */
    public static double PROB_CROSSOVER_UNIFORM = 0.8;

    /**
     * The probability of mutating one allele and the action in an offspring classifier.
     */
    public static double PROB_MUTATION_MU = 0.04;
    
    /**
     * The probability of mutating one allele in the feature.
     */
    public static double PROB_MUTATION_UNIFORM = 0.8;

    /**
     * The probability of using a don't care symbol in an allele when covering.
     */
    public static double PROB_DONTCARE = 0.5;

    /**
     * The reduction of the prediction error when generating an offspring classifier.
     */
    public static double PREDICTION_ERROR_REDUCTION = 0.25;

    /**
     * The reduction of the fitness when generating an offspring classifier.
     */
    public static double FITNESS_REDUCTION = 0.1;

    /**
     * The experience of a classifier required to be a subsumer.
     */
    public static int SUBSUMPTION_EXP_THRESHOLD_THETA_SUB = 20;

    /**
     * Specifies if action set subsumption should be executed.
     */
    public static boolean FLAG_ACTION_SET_SUBSUMPTION = true;

    /**
     * The initial prediction value when generating a new classifier (e.g in covering).
     */
    public static double INITIAL_PREDICTION = 10.0;

    /**
     * The initial prediction error value when generating a new classifier (e.g in covering).
     */
    public static double INITIAL_PREDICTION_ERROR = 0.0;

    /**
     * The initial prediction value when generating a new classifier (e.g in covering).
     */
    public static double INITIAL_FITNESS = 0.01;




}
