package fpcs.listeners;

import fpcs.core.ClassifierSet;
import fpcs.core.ITrainingListener;
import fpcs.core.State;
import fpcs.utils.BagOfValues;
import java.util.Arrays;


public class ConfusionMatrixListener implements ITrainingListener {

    private int[] confusions;
    private int numActions;
    private int[] totals;
    private int startGeneration = 0;
    private int counter = 0;
    public int printPeriod = 0;


    public ConfusionMatrixListener(int numActions) {
        this.numActions = numActions;
        confusions = new int[numActions * numActions];
        totals = new int[numActions];
    }

    public ConfusionMatrixListener(int numActions, int startGeneration) {
        this(numActions);
        this.startGeneration = startGeneration;
    }


    @Override
    public void onExploit(int generation, State state, int actualAction, ClassifierSet population, ClassifierSet matchSet, int actionWinner, double prediction, double reward, boolean wasCorrect) {
        if (counter >= startGeneration) {

            int stateAction =actualAction;
            int agentAction = actionWinner;

            recordAction(stateAction, agentAction);

            if (printPeriod > 0 && counter % printPeriod == 0) {
                printMatrix();
            }

        }
        counter++;
    }

    public void recordAction(int actualAction, int agentAction) {
        confusions[actualAction * numActions + agentAction] += 1;
        totals[actualAction] += 1;
    }

    public void printMatrix() {

        System.out.print("|  E\\A  |");
        for (int i = 0; i < numActions; i++) {
            System.out.print("   " + i + "   |");
        }
        System.out.println("");

        for (int i = 0; i < numActions; i++) {
            System.out.print("|   " + i +"   |");

            for(int j = 0; j < numActions; j++) {
                System.out.print(" " + String.format("%.3f", confusions[i * numActions + j] / (double)totals[i]) + " |");
            }
            System.out.println("");
        }

    }

    /**
     *
     * @param errorArray
     * @return sum of errors
     */
    public double fillErrorArray(double[] errorArray) {

        assert(errorArray.length == numActions);

        double sum = 0;

        for(int i = 0; i < numActions; i++) {
            if (totals[i] > 0) {
                errorArray[i] = 1.1 - (confusions[i * numActions + i] / (double)totals[i]); // at least 0.1
            } else {
                errorArray[i] = ((double)numActions - 1.0) / numActions; // random guess
            }
            sum += errorArray[i];
        }

        return sum;

    }

    public double getError(int actualAction, int agentAction) {
        return (double)confusions[actualAction * numActions + agentAction] / totals[actualAction];
    }

    public void reset() {
        Arrays.fill(totals, 0);
        Arrays.fill(confusions, 0);
        counter = startGeneration;
    }


    public int getTotalCalls() {
        return counter - startGeneration;
    }
}
